import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, notification } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import MKEmitter from '@/utils/events.js'
import './index.scss'

class SandBoxComponent extends Component {
  static propTpyes = {
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
  }

  state = {
    BID: '',                   // 上级ID
    config: null,              // 图表配置信息
    loading: false,            // 数据加载状态
    sync: false,               // 是否统一请求数据
    data: [],                  // 数据
    html: '',
    result: {}
  }

  loaded = false

  UNSAFE_componentWillMount () {
    const { data, initdata } = this.props
    let _config = fromJS(this.props.config).toJS()

    let _data = []
    let _sync = false

    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }
    
    if (_config.wrap.datatype !== 'static') {
      _sync = _config.setting.sync === 'true'

      if (_sync && data) {
        _data = data[_config.dataName] || []
        _sync = false
        this.loaded = true
      } else if (_sync && initdata) {
        _data = initdata || {}
        _sync = false
        this.loaded = true
      }
    } else {
      _data = {}
      this.loaded = true
    }
    
    if (_config.css) {
      let node = document.getElementById(_config.uuid)
      node && node.remove()

      let ele = document.createElement('style')
      ele.id = _config.uuid
      ele.innerHTML = _config.css
      document.getElementsByTagName('head')[0].appendChild(ele)
    }

    this.setState({
      sync: _sync,
      data: _data,
      BID: BID || '',
      config: _config,
      arr_field: _config.columns.map(col => col.field).join(','),
    }, () => {
      if (_config.wrap.datatype !== 'static' && _config.setting && _config.setting.sync !== 'true' && _config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, _config.setting.delay || 0)
      }
      this.renderView()
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)

    if (config.$cache && !this.loaded) {
      Api.getLCacheConfig(config.uuid).then(res => {
        if (!res || this.loaded) return

        this.setState({data: res}, () => {
          this.renderView()
        })
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
  }

  /**
   * @description 图表数据更新，刷新内容
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = []
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName]
      }

      this.loaded = true

      if (!is(fromJS(this.state.data), fromJS(_data))) {
        setTimeout(() => {
          this.renderView()
        }, 10)
      }

      this.setState({sync: false, data: _data})
    } else if (config.setting.useMSearch && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({}, () => {
        this.loadData()
      })
    }
  }

  reloadData = (menuId) => {
    const { config } = this.state

    if (menuId !== config.uuid) return

    this.loadData()
  }

  async loadData () {
    const { mainSearch } = this.props
    const { config, arr_field, BID } = this.state

    if (config.wrap.datatype === 'static') {
      this.setState({
        data: [],
        loading: false
      })
      return
    } else if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: [],
        loading: false
      })
      this.loaded = true
      return
    }

    let searches = config.setting.useMSearch && mainSearch ? mainSearch : []

    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    this.setState({
      loading: true
    })

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = result.data || []

      this.loaded = true
      if (config.$cache && config.setting.onload !== 'false') {
        Api.writeCacheConfig(config.uuid, result.data || '')
      }

      if (!is(fromJS(this.state.data), fromJS(_data))) {
        setTimeout(() => {
          this.renderView()
        }, 10)
      }

      let _result = {...result}
      delete _result.data

      this.setState({
        data: _data,
        result: _result,
        loading: false
      })
    } else {
      this.setState({
        loading: false
      })
      notification.error({
        top: 92,
        message: result.message,
        duration: 10
      })
    }
  }

  renderView = () => {
    const { data, result } = this.state
    const { html, js, wrap, columns } = this.state.config

    let _html = html
    if (_html && wrap.datatype !== 'static') {
      if (/@[\u4E00-\u9FA50-9a-zA-Z_]+@/ig.test(_html)) {
        let _data = data[0] || {}

        columns.forEach(col => {
          if (col.field) {
            let val = _data[col.field] !== undefined ? _data[col.field] : ''
            let reg = new RegExp('@' + col.field + '@', 'ig')
  
            _html = _html.replace(reg, val)
          }
        })
      }
    }

    this.setState({html: _html}, () => {
      if (js) {
        try {
          // eslint-disable-next-line
          let evalfunc = eval('(true && function (data, result) {' + js + '})')
          evalfunc(data, result)
        } catch (e) {
          console.warn(e)
        }
      }
    })
  }

  render() {
    const { config, loading, html } = this.state

    return (
      <div className="custom-sand-box" id={'anchor' + config.uuid} style={{...config.style}}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        <div dangerouslySetInnerHTML={{ __html: html }}></div>
      </div>
    )
  }
}

export default SandBoxComponent